#! /usr/bin/perl
#
# $Header: //sapdb/TOOLSRC/develop/sys/src/BuildPackage/WinLink.pm#2 $
# $DateTime: 2002/06/10 14:15:26 $
# $Change: 22128 $
#
# Desc:
#
#    ========== licence begin LGPL
#    Copyright (C) 2002 SAP AG
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#    ========== licence end
#

#
# A few basics right at the beginning:
close STDERR;
open STDERR, ">&STDOUT";

$| = 1; # Nur zur Sicherheit, damit das (unntige) Buffering ausgeschaltet wird.


if ($^O =~ /MSWin32/i) {
	require File::DosGlob ;
	import  File::DosGlob 'glob';
	require WinLink;
	import  WinLink;
	require Win32::TieRegistry;
	import  Win32::TieRegistry;
	require Win32::Process;
	import  Win32::Process;
}

use testdb;
my %options = ();
use Getopt::Long;
if (! &GetOptions(	\%options,
			'h|help|?',
			'package=s@',
			'testonly',
			'verbose|V',
			'test_profile=s',
			'sutonly',
			'xterm',
			'testname=s',
			'dbname=s',
			'delay=i',
			'cmd=s',
			'testgroups|nog=i'
			)) {
	print("Run testdb.pl -h for help");
	exit(1);
}

if (($options{'h'})) #||(!$options{'version'}) || (!$options{'status'})) { 
{
	print <<EOF;

	Usage:
testdb.pl -package <VERSION><STATUS> [-verbose] [-xterm] 
          [ -verbose ] [ -noinstall ] [ -test_profile <PROFILENAME> ] 

where <VERSION> can either be
	7403	(This can be a possible exception on AIX 4)
or
	7402

and <STATUS> can be
	DEV
or
	COR
or
	RAMP
	
<PROFILENAME> can be
	workday	(the default)
or
	weekend
	
-verbose   tells you a bit more of what the script is doing. 
-xterm     will open an xterm-Windows with the right environment.
           Please make sure that your DISPLAY-Varible is set correctly.
-testname  Will perform a single test without any instalation. You 
           need to provide the dbname parameter with this.
-dbname    set the name of the DB-Instance for this test. It will be 
           enhanced with Version and QA-Informations.

EOF
exit(0)
}

if ((defined ($options{'testgroups'})) and (!(defined $options {'delay'}))) {
	print "Error in option-Mix: you gave me a 'testgroups'-parameter but no 'delay'-parameter.Exiting\n";
	exit(1);
}

if ($options{'test_profile'} ne "weekend") {
		$options{'test_profile'} = "workday";
}
	
my @testq;

########
# Anayzing arguments for tests to be performed.
########
foreach $x   (@{$options{'package'}}) {
	if ($x =~ /^\d{4}(DEV|COR|RAMP)$/i) {
		push(@testq, testdb->new({'version' => substr($x, 0, 4), 'status' => substr($x, 4, 10), 'profile' => $options{'test_profile'}}));
		if ($testq[-1]->{error_code}) {
			testdb::throw_err ($testq[-1]->{error_text});
			pop(testq);
		}
		print substr($x, 0, 4) . " " . substr($x, 4, 10) . "\n";
	}
	else {
		testdb::throw_err("testdb.pl got wrong parameters :-((");
	}
}


if (defined ($options{'testname'})) {
	if (!(defined ($options{'dbname'}))) {
		print "You need to provide a DB-Name if you want to perform a single Test\nCall testdb.pl -h for further informations\n";
		exit(1);
	}

	$x = $testq[0];
	$x->run_single_test($options{'testname'}, $options{'dbname'});
	exit(0);
}


if (defined ($options{'xterm'})) {
	$x = $testq[0];
	$x->term();
	exit(0);
}

if (defined ($options{'cmd'})) {
	$x = $testq[0];
	$x->cmd($options{'cmd'});
	exit(0);
}

if (defined($options{'testonly'})) {
	foreach $x (@testq) {
		$x->runTF();
	}

}
elsif (defined($options{'sutonly'})) {
	foreach $x (@testq) {
		$x->runSUT();
	}
}
else {
	my $upcount = 0;
	foreach $x (@testq) {
		$x->preClean();
		$x->installLC();
		$x->installTF();
	}
	
	if ($^O =~ /MSWin32/i) {
		my @process_id;
		foreach $x (@testq) {
			sleep(30);
			push(@process_id, "");
			print "perl D:\\SAP_DB\\TESTDB\\testdb.pl  -test_profile $options{'test_profile'} -testonly -package " . $x->{'version'} . $x->{'status'};
			Win32::Process::Create($process_id[-1],
					"D:\\devtool\\Perl\\bin\\perl.exe",
					"perl D:\\SAP_DB\\TESTDB\\testdb.pl -test_profile $options{'test_profile'} -testonly -package " . $x->{'version'} . $x->{'status'},
					0,
					NORMAL_PRIORITY_CLASS,
					"D:\\SAP_DB\\TESTDB") || 
					print("Error while trying to execute sub-perl:\n" . Win32::FormatMessage(Win32::GetLastError()));
			push(@process_id, "");
			sleep(30);
			Win32::Process::Create($process_id[-1],
					"D:\\devtool\\Perl\\bin\\perl.exe",
					"perl D:\\SAP_DB\\TESTDB\\testdb.pl   -sutonly -package " . $x->{'version'} . $x->{'status'},
					0,
					NORMAL_PRIORITY_CLASS,
					"D:\\SAP_DB\\TESTDB") || 
					print("Error while trying to execute sub-perl:\n" . Win32::FormatMessage(Win32::GetLastError()));
					
			
			$upcount++;
			print "Delay = $options{'delay'} \n";
			print "Testgroups = $options{'testgroups'} \n";
			if (defined ($options{'testgroups'}) ) {
				if ($upcount == $options{'testgroups'}) {
					$upcount = 0;
					sleep ($options{'delay'});
				}
			}
		}
		foreach $x (@process_id) {
			$x->Wait(INFINITE);
		}
	}
	else {
		my @process_id;
		foreach $x (@testq) {
			
			$x->unlock();
			$rv = fork();
			$x->lock();
			
			if ($rv) {
				print "New process ID created;: $rv\n";
				push(@process_id, $rv);
			}
			else {
				$x->runTF();
				exit(0);
			}
			sleep (30);
			
			$x->unlock();
			$rv = fork();
			$x->lock();
			
			if ($rv) {
				push(@process_id, $rv);
			}
			else {
				$x->runSUT();
				exit(0);
			}
			sleep (30);
			$upcount++;
			print "Delay = $options{'delay'} \n";
			print "Testgroups = $options{'testgroups'} \n";
			if (defined ($options{'testgroups'}) ) {
				if ($upcount == $options{'testgroups'}) {
					$upcount = 0;
					sleep ($options{'delay'});
				}
			}
		}
		foreach $x (@process_id) {
			waitpid ($x, 0);
		}
	}

}

