#! /usr/bin/perl
#
#    ========== licence begin LGPL
#    Copyright (C) 2002 SAP AG
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#    ========== licence end
#

package testdb;

@ISA = ('Exporter');
@EXPORT = ('new');


BEGIN {
	push(@INC, "/devtool/local/bin");
	push(@INC, "/devtool/TOOL/tool/bin");
	push(@INC, "/devtool/TOOL/tool/lib/perl5");
	push(@INC, "/devtool/TOOL/tool/lib");
	push(@INC, "/SAP_DB/TESTDB/lib");

}
use Getopt::Long;
use Sys::Hostname;
use File::Copy;
use qadb;

if ($^O =~ /MSWin32/i) {
	require File::DosGlob ;
	import  File::DosGlob 'glob';
	require WinLink;
	import  WinLink;
	require Win32::TieRegistry;
	import  Win32::TieRegistry;
	require Win32::Process;
	import  Win32::Process;
}
1;

sub new{
	#
	# Lets get initialized
	#
	my $name = shift;
	my $options = shift;
	
	my $self = {};
	$self->{'cmdcount'};
	$self->{'path'} = $ENV{'PATH'};
	for $x (keys(%{$options})) {
		print "$x    => ${$options}{$x}\n";
	}
	
	##
	## Basic settings: things we should know about:
	##
	$self->{'versions'}    = ['7402', '7403', '7404'];
	$self->{'status'}      = ['DEV', 'COR', 'RAMP'];
	$self->{'profiles'}    = ['workday', 'weekend'];	
	$self->{'hostname'}    = hostname();
	$self->{'error_text'}  = "";
	$self->{'error_code'}  = 0;
	$self->{'prot_count'}  = 0;

	if ($^O =~ /MSWin32/i) {
		$self->{'delimit'}       = "\\"; # As we know, Windows uses backslashes
		$self->{'pathsep'}       = ";";
		$self->{'rootdir'}       = "D:\\SAP_DB\\";
		$self->{'rmcmd'}         = "del /s /q ";
		$self->{'xserver_param'} = "";
		$self->{'path'}          = $self->{'path'} . ";C:\WINNT;C:\WINNT\system32;D:\DEVTOOL\bin;d:\devtool\adminbin;d:\devtool\pgm;d:\devtool\posix;d:\devtool\Perl;d:\devtool\python\bin";
		$ENV{'JTEST_TOOL'} = "d:\\devtool\\";
		$ENV{'TOOLVARS'} = "d:\\devtool\\bin\\toolvars.pl";
		$ENV{'TOOL'} = "d:\\devtool";
		$ENV{'TOOLEXT'} = ".pl";
		$ENV{'TOOLSHELL'} = "d:\\devtool\\perl\\bin\\perl";
		$ENV{'PYTHON'} = "D:\\devtool\\python";
		$ENV{'PERL'} = "d:\\devtool\\perl";
		$ENV{'PERL5LIB'} = $ENV{'TOOL'} . "\\bin;" . $ENV{'TOOL'} ."\\perl;" . $ENV{'TOOL'} ."\\Perl\\site;" . $ENV{'TOOL'} ."\\Perl\\site\\lib";
		$ENV{'ISWDFNACHT'}  = 1;
	}
	else {
		$self->{'delimit'}       = "/";
		$self->{'pathsep'}       = ":";
		$self->{'rootdir'}       = "/SAP_DB/";
		$self->{'user'}          = "remuser";
		$self->{'group'}         = "sapsys";
		$self->{'instwrap'}      = $self->{'rootdir'} . "TESTDB/tinysudo";
		$self->{'rmcmd'}         = "rm -rf ";
		$self->{'vserverext'}    = ".old";
		$self->{'ininame'}       =  "/usr/spool/sql/ini/SAP_DBTech.ini";
		$self->{'xserver_param'} = " -Y";
		$self->{'path'}         .= ":/devtool/TOOL/tool/pgm:/devtool/TOOL/tool/Posix:/usr/bin/X11:.";
		$self->{'path'}         .= ":/usr/bin:/bin:/usr/local/bin:/devtool/local/bin:/devtool/TOOL/tool/bin";
		$ENV{'TOOL'}             = "/devtool/TOOL/tool";
		$ENV{'TOOLEXT'}          = ".pl";
		$ENV{'TOOLSHELL'}        = "/devtool/TOOL/tool/Posix/perl";
		$ENV{'PYTHON'}           = "/devtool/TOOL/tool/Python";
		$ENV{'PERL'}             = "/devtool/local";
		$ENV{'PERL5LIB'}         = $ENV{'TOOL'} . "/bin:" . $ENV{'TOOL'} . "/lib/perl5";
		$ENV{'RELVER'}           = "R74"; # Well, this will have to be taken into getVersionDep someday.
		$ENV{'JTEST_TOOL'}       = $ENV{'TOOL'};
		$ENV{'TOOLVARS'}         = $ENV{'TOOL'} . "/bin/toolvars";
		$ENV{'BIT64'}            = '1'; # Someday, this belongs to getHostDep
		$ENV{'ISWDFNACHT'}       = '1';
	}
	
	print $options{'status'} . $options{'version'} . "\n";
	##################################################################
	# parameter checking
	##################################################################
	for $x (@{$self->{'status'}}) {
		print "$x\n";
	}
	if ((grep /^$options{'status'}$/, @{$self->{'status'}}) and (grep /^$options{'version'}$/, @{$self->{'versions'}})) {
		print "dd\n";
		$self->{'error_text'} = "Wrong parameters";
		$self->{'error_code'} = 1;
		return bless $self;
	}

	$self->{'version'} = ${$options}{'version'};
	$self->{'status'}  = ${$options}{'status'};
	if (${$options}{'profile'} eq 'weekend') {
		$self->{'profile'} = 'weekend';
	}
	else {
		$self->{'profile'} = 'workday';
	}

	$self->{'hostname'} = hostname();
	
	if ($self->{'hostname'} eq "us0062") {
		$self->{'path'}     .= ":/usr/j2se/jre/bin:/sapdb/programs/bin:/sapdb/programs/pgm:/opt/WS6U2/SUNWspro/bin:/opt/WS6U2/SUNWspro";
		$self->{'bits'}      = "64";
		$self->{'platform'}  = "sun_64";
	}
	elsif ($self->{'hostname'} eq "ds0116") {
		$self->{'path'}     .= ":/usr/opt/java131/bin:/SAP_DB/TESTDB/prog_indep/bin:/SAP_DB/TESTDB/prog_indep/pgm";
		$self->{'bits'}      = "64";
		$self->{'platform'}  = "alphaosf";
	}
	elsif ($self->{'hostname'} eq "is0025") {
		$self->{'path'}      = ":/usr/java130/jre/sh:/SAP_DB/programs/bin:/SAP_DB/programs/pgm";
		$self->{'bits'}      = "64";
		$self->{'platform'}  = "rs6000_51_64";
	} 
	elsif ($self->{'hostname'} eq "is0026") {
		$self->{'path'}      = ":/usr/java131/jre/sh:/SAP_DB/programs/bin:/SAP_DB/programs/pgm";
		$self->{'bits'}      = "64";
		$self->{'platform'}  = "rs6000_64";
		$self->{'xserver_param'} = ""; # This is secialy needed for this release
		pop @versions; # Currently, SAP DB 7.4.3 is only availble von AIX 5, not an AIX 4
	}
	elsif ($self->{'hostname'} eq "hs0102") {
		$self->{'path'}    .= ":/opt/java1.3/jre/bin:/SAP_DB/programs/bin:/SAP_DB/programs/pgm";
		$self->{'bits'}     = "64";
		$self->{'platform'} = "hp_64";
		$ENV{'UNIX95'}      = "1";
		$ENV{'TZ'}          = "MET-1METDST";
	}
	elsif ($self->{'hostname'} eq "PWDF0238.wdf.sap.corp") {
		$self->{'bits'}     = "32";
		$self->{'platform'} = "MSWin32";
	}
	elsif ($self->{'hostname'} eq "itanium21.nt5.sap-ag.de") {
		$ENV{'BIT64'}      = '1';
		$self->{'bits'} = "64";
		$self->{'platform'} = "MSWin32";
		
	}
	else {
		print "The host $self->{'hostname'} ist not known in the configuration\n(See sub getdep!)\n";
		exit(1);
	}
	
	$self->{'qah'} = qadb->new_test({'PLATFORM' => $self->{'platform'}, 'QASTATUS' => $self->{'status'}, 'VERSION' => $self->{'version'}});
	if ($qah->{'error_code'} != 0) {
		print "Exiting:\n" . $qah->{'error_text'} . "\n";
		exit(1);
	}
	
	
	$ENV{'PLATFORM'} = $self->{'platform'};
	
	$self->{'node'} = $self->{'version'} . $self->{'status'};
	
	
	##################################################################
	# Find out which files to use
	##################################################################
	if (($options->{'status'} eq "RAMP") && !($^O =~ /MSWin32/i)){
		@subdirs           = glob ("/bas/SAP_DB/${self->{'version'}}/pkg/$self->{'platform'}/LC_${self->{'version'}}??_${self->{'bits'}}_RAMP");
		sort (@subdirs);
		$self->{'subdir'}  = pop(@subdirs);
		
		@files             = (glob "$self->{'subdir'}/" . $self->{'qah'}->{'LCPOOLID'} . "/SAPDB$self->{'version'}*.SAR");
		$self->{'srcfile'} = pop(@files);
		
		$self->{'srcdir'}  = "$self->{'subdir'}/" . $self->{'qah'}->{'LCPOOLID'};
		$self->{'build'}   = substr($self->{'subdir'}, length("/bas/SAP_DB/$self->{'version'}/pkg/$self->{'platform'}/LC_$self->{'version'}"), 2);
		$self->{'sdbinst'} = $self->{'srcdir'} . "/SAPDB_COMPONENTS/SDBINST";
	}
	
	elsif (($options->{'status'} eq "RAMP") && ($^O =~ /MSWin32/i)) {
		@subdirs = glob ("\\\\pwdfm017\\LC_POOL\\LC_$self->{'version'}??_$self->{'bits'}_RAMP");
		sort (@subdirs);
		$self->{'subdir'}  = pop(@subdirs);
		$self->{'srcdir'}  = $self->{'subdir'} . "\\" . $self->{'qah'}->{'LCPOOLID'};
		@files             = (glob ("$self->{'srcdir'}\\SAPDB$self->{'version'}*.SAR"));
		$self->{'srcfile'} = pop(@files);
		$self->{'build'}   = substr($self->{'subdir'}, length("\\\\pwdfm017\\LC_POOL\\LC_$self->{'version'}"), 2);
		$self->{'sdbinst'} = $self->{'srcdir'} . "\\SAPDB_COMPONENTS\\SDBINST.EXE";
	}
	
	elsif (($options->{'status'} ne "RAMP") && !($^O =~ /MSWin32/i)) {
		$self->{'subdir'} = "/bas/SAP_DB/$self->{'version'}/pkg/$self->{'platform'}/LC_$self->{'version'}_$self->{'bits'}_$self->{'status'}";
		@files = (glob "$self->{'subdir'}/LastBuild/SAPDB$self->{'version'}*.SAR");
	
		$self->{'srcfile'} = pop(@files);
		$self->{'srcdir'}  = "$self->{'subdir'}/LastBuild";
		$self->{'build'}   = substr($self->{'srcfile'}, -6 ,2);
		$self->{'sdbinst'} = $self->{'srcdir'} . $self->{'delimit'} . "SAPDB_COMPONENTS" . $self->{'delimit'} . "SDBINST";
	}
	elsif (($options->{'status'} ne "RAMP") && ($^O =~ /MSWin32/i)) {
		$self->{'subdir'}   = "\\\\pwdfm017\\LC_POOL\\LC_$self->{'version'}_$self->{'bits'}_$self->{'status'}";
		$self->{'srcdir'}   = $self->{'subdir'} . "\\" . $self->{'qah'}->{'LCPOOLID'};

		@files              = (glob "$self->{'srcdir'}\\SAPDB$self->{'version'}*.SAR");
		$self->{'srcfile'}  = pop(@files);
		$self->{'build'}    = substr($self->{'srcfile'}, -6, 2);
		$self->{'sdbinst'}  = $self->{'srcdir'} . "\\SAPDB_COMPONENTS\\SDBINST.EXE";	
	}
	
	if (-e ($self->{'srcdir'} . $self->{'delimit'} . "changelist.log")) {
		
		open (CLFD, $self->{'srcdir'} . $self->{'delimit'} . "changelist.log");
		while (<CLFD>) {
			if (/\d{5}/) {
				$self->{'changelist'} = $_;
				chomp $self->{'changelist'};
			}
		}
	}

	
	$self->{'tempdir'}    = $self->{'rootdir'} . $self->{'node'} . $self->{'delimit'} . "tmp";
	$self->{'dbroot'}     = $self->{'rootdir'} . $self->{'node'} . $self->{'delimit'} . "db";
	$self->{'testdir'}    = $self->{'rootdir'} . $self->{'node'} . $self->{'delimit'} . "test";
	$self->{'jtest_root'} = $self->{'testdir'} . $self->{'delimit'}. "jtest";

	# Set the Path to use the right directorys:
	

	
	#
	# Read the independend program path
	#
	
	open (DBMCLII, "dbmcli -s dbm_getpath IndepProgPath|");
	while (<DBMCLII>) {
		if (!(/^OK/)) {
			$self->{'indeppath'} = $_;
		}
	}
	chomp($self->{'indeppath'});
	$ENV{'INDEPPATH'}   = $self->{'indeppath'};
	$self->{'vserver'}  = $self->{'indeppath'} . $self->{'delimit'} . "pgm" . $self->{'delimit'} . "vserver";

	$self->{'path'}    .= $self->{'testdir'} . $self->{'delimit'} . "pc" . $self->{'delimit'} . "bin" . $self->{'pathsep'} . $self->{'dbroot'} . $self->{'delimit'} . "bin". $self->{'pathsep'} . $self->{'dbroot'} . $self->{'delimit'} . "pgm";
	$ENV{'HOME'}        = $self->{'tempdir'}; # For the PC-Tests.
	$ENV{'JTEST_ROOT'}  = $self->{'jtest_root'};
	$ENV{'INSTROOT'}    = $self->{'dbroot'};
	$ENV{'TEST_ROOT'}   = $self->{'testdir'};
	$ENV{'TESTROOT'}    = $self->{'testdir'};
	$ENV{'SAPDBSDK'}    = $self->{'dbroot'} . $self->{'delimit'} . "sdk" . $self->{'delimit'} . $self->{'version'};
	$ENV{'TMP'}         = $self->{'tempdir'};
	$ENV{'TEMP'}        = $self->{'tempdir'};

		for $x (keys(%$self)) {
		print "$x = " . $self->{$x} . "\n";
	}
	return bless $self;
}

sub unlock {
	my $self = shift;
	
	return $self->{'qah'}->unlock();
}

sub lock {
	my $self = shift;
	
	$self->{'qah'} = $self->{'qah'}->lock();
	
	return 0;
}
#
# Provide a terminal
#
sub term {
	$self = shift;
	$ENV{'PATH'}        = $self->{'path'};
	$ENV{'HOME'}        = $self->{'tempdir'}; # For the PC-Tests.
	$ENV{'JTEST_ROOT'}  = $self->{'jtest_root'};
	$ENV{'INSTROOT'}    = $self->{'dbroot'};
	$ENV{'TEST_ROOT'}   = $self->{'testdir'};
	$ENV{'TESTROOT'}    = $self->{'testdir'};
	$ENV{'SAPDBSDK'}    = $self->{'dbroot'} . $self->{'delimit'} . "sdk" . $self->{'delimit'} . $self->{'version'};
	$ENV{'TMP'}         = $self->{'tempdir'};
	$ENV{'TEMP'}        = $self->{'tempdir'};
	$ENV{'CORRECTION_LEVEL'} = substr($self->{'version'}, 2, 2);

	if ($^O =~ /MSWin32/i) {
		system ("cmd");
	}
	else {
		system ("xterm &");
	}
}

sub cmd {
	$self = shift;
	$cmd  = shift;
	
	$ENV{'PATH'}        = $self->{'path'};
	$ENV{'HOME'}        = $self->{'tempdir'}; # For the PC-Tests.
	$ENV{'JTEST_ROOT'}  = $self->{'jtest_root'};
	$ENV{'INSTROOT'}    = $self->{'dbroot'};
	$ENV{'TEST_ROOT'}   = $self->{'testdir'};
	$ENV{'TESTROOT'}    = $self->{'testdir'};
	$ENV{'SAPDBSDK'}    = $self->{'dbroot'} . $self->{'delimit'} . "sdk" . $self->{'delimit'} . $self->{'version'};
	$ENV{'TMP'}         = $self->{'tempdir'};
	$ENV{'TEMP'}        = $self->{'tempdir'};
	$ENV{'CORRECTION_LEVEL'} = substr($self->{'version'}, 2, 2);

	
	system($cmd);
}

##################################################################
# subs
##################################################################
##################################################################
# preClean:
# Throws away current instances, unregisters the installation 
# and eraes tempoary and test directorys.
##################################################################
sub preClean {
	$self = shift;
	$ENV{'PATH'}        = $self->{'path'};
	$ENV{'HOME'}        = $self->{'tempdir'}; # For the PC-Tests.
	$ENV{'JTEST_ROOT'}  = $self->{'jtest_root'};
	$ENV{'INSTROOT'}    = $self->{'dbroot'};
	$ENV{'TEST_ROOT'}   = $self->{'testdir'};
	$ENV{'TESTROOT'}    = $self->{'testdir'};
	$ENV{'SAPDBSDK'}    = $self->{'dbroot'} . $self->{'delimit'} . "sdk" . $self->{'delimit'} . $self->{'version'};
	$ENV{'TMP'}         = $self->{'tempdir'};
	$ENV{'TEMP'}        = $self->{'tempdir'};
	$ENV{'CORRECTION_LEVEL'} = substr($self->{'version'}, 2, 2);

        &doCmd($self, "x_niserver stop");
        &doCmd($self, "x_server $self->{'xserver_param'}");
	open (DBLIST, 'dbmcli db_enum|');
	my $x = "";
	while (<DBLIST>) {
		if ($self->{'verbose'}) {
			print "$_ \n";
			print $self->{'node'} . "\n";
		}
		
		if ( /fast/ && /$self->{node}/i ) {
			s/^(\w+)\s+.*$/$1 /;
			$x = $_;
			chomp($x);
			foreach $y ("control,control", "dbm,dbm", "superdba,colduser") {
				print "dbmcli -u $x -d $x\n";
				&doCmd($self, "dbmcli -u $y -d $x db_stop");
				&doCmd($self, "dbmcli -u $y -d $x db_clear");
				&doCmd($self, "dbmcli -u $y -d $x db_drop");
			}
		}
	}

	&doCmd($self, "x_niserver stop");
	print "#######################################################################################\n";
	&doCmd($self, "x_server stop");
	
	# If we were called with -noinstall, it is _not_ usefull to 
	# deregister the installation.
	if (!($options{'noinstall'})) {
		&doCmd($self, "dbmcli inst_unreg $self->{'dbroot'}");
	}
	
	if ($^O =~ /MSWin32/i) {
		&doCmd($self, "kill dbmsrv.exe");
	}
	
	
	&doCmd($self, "$self->{'rmcmd'} $self->{'tempdir'}" . $self->{'delimit'} . "*");
	&doCmd($self, "$self->{'rmcmd'} $self->{'testdir'}" . $self->{'delimit'} . "* ");
}

##################################################################
# installLC:
# installs the LC depending on the informations we have
# already figured out.
##################################################################
sub installLC {
	$self = shift;

	if ($^O =~ /MSWin32/i) {
		system("x_server stop");
		
		system("$self->{'sdbinst'}  -b -profile Server -depend $self->{'dbroot'} ");
		system("x_server $self->{'xserver_param'}");
	}
	else {
	        system("x_server stop");

		# Set the lastTest-Directory correctly:
		# First, the old LastTest has to be unlinked
		if (-e $self->{'subdir'} . $self->{'delimit'} . "LastTest") {
			if (!($options{'dry'})) {
				if ($options{'verbose'}) {
					print "Unlinking LastTest $self->{'subdir'}" . $self->{'delimit'} . "LastTest\n"
				}
				unlink ($self->{'subdir'} . "" . $self->{'delimit'} . "LastTest") || print "Could not erase LastTest!\n";
			}
			else {
				print "Normaly, the LastTest ($self->{'subdir'}" . $self->{'delimit'} . "LastTest) would be erased now.\n";
			}
		}
		# then, create a new link
		if (!($options{'dry'})) {
			if ($options{'verbose'}) {
				print "Linking $self->{'subdir'}" . $self->{'delimit'} . "$self->{'qah'}->{'LCPOOLID'} to LastTest\n";
			}
			symlink ($self->{'subdir'} .  $self->{'delimit'} . $self->{'qah'}->{'LCPOOLID'}, $self->{'subdir'} . $self->{'delimit'} . "LastTest") || 
				print "Could not link $self->{'subdir'}" . $self->{'delimit'} . $self->{'qah'}->{'LCPOOLID'} . " to LastTest\n";
		}
		else {
			print "Normaly:\nLinking $self->{'subdir'}" . $self->{'delimit'} . "$self->{'qah'}->{'LCPOOLID'} to LastTest\n";
		}
		# End of link-setting
		
	
	

		# perform the installation itself
		$self->{'inst_param'} = " -b -profile Server -o $self->{'user'} -g $self->{'group'} -depend $self->{'dbroot'} ";
		$self->{'odbcinst_param'} = " -b -profile ODBC -o $self->{'user'} -g $self->{'group'}  ";
		&doCmd($self, " $self->{'instwrap'} $self->{'sdbinst'} $self->{'inst_param'}");
	
	}
	
	print "#####ODBC###############################################################\n";
	#####ODBC###############################################################
	# Okay, dann machen wir mal etwas ODBC
	# First, remove the currently installed ODBC package
	&doCmd($self, "echo y | $self->{'instwrap'} $self->{'indeppath'}" . $self->{'delimit'} . "bin" . $self->{'delimit'} . "sdbuninst -package ODBC");
	# Then install the package we want.
	&doCmd($self, " $self->{'instwrap'} $self->{'sdbinst'} $self->{'odbcinst_param'} ");
	
	
	
	#####PRECOMPILER###############################################################
	# Okay, now we will handle the precompiler
	#
	
	print "#####PRECOMPILER###############################################################\n";
	&doCmd($self, "cd $self->{'dbroot'} && SAPCAR -xvf $self->{'srcdir'}" . $self->{'delimit'} . "SAPDBCSDK.SAR");

	##### RESTART THE x_server
	&doCmd($self, "x_server $self->{'xserver_param'}");		
	
	#####SIMULATOR###############################################################
	# Okay, now we will install the files for the lc-simulator
	#
	
 
	print "#####SIMULATOR###############################################################\nStart copying simulator files:\n";
	
	open LIST_FH, ($self->{'srcdir'} . $self->{'delimit'}. "simul.lst") or return throw_err("Could not open $self->{'srcdir'}$self->{'delimit'}simul.lst\n");
	
	while(<LIST_FH>) {
		print "###########################################\nTry to copy ${_}: \n";
		
		# At first: Figure out what the sourcename is
		my $srcfile  = "$self->{'srcdir'}$self->{'delimit'}sys$self->{'delimit'}src$self->{'delimit'}lcsrc$self->{'delimit'}$_";
		# Then find the target-directory:
		
		my $destfile = "$self->{'dbroot'}$self->{'delimit'}$_";
		chomp $srcfile;
		chomp $destfile;
		print "$srcfile -> $destfile\n";
		copy ($srcfile, $destfile) or throw_err ("Could not copy $srcfile to $destfile:\n$! \n");
		chmod ((stat($srcfile))[2], $destfile);

	}
}

##################################################################
# installTF:
# installs the Java TestFrame depending on the informations we 
# have already figured out.
##################################################################
sub installTF {
	$self = shift;
	$ENV{'PATH'}        = $self->{'path'};
	$ENV{'HOME'}        = $self->{'tempdir'}; # For the PC-Tests.
	$ENV{'JTEST_ROOT'}  = $self->{'jtest_root'};
	$ENV{'INSTROOT'}    = $self->{'dbroot'};
	$ENV{'TEST_ROOT'}   = $self->{'testdir'};
	$ENV{'TESTROOT'}    = $self->{'testdir'};
	$ENV{'SAPDBSDK'}    = $self->{'dbroot'} . $self->{'delimit'} . "sdk" . $self->{'delimit'} . $self->{'version'};
	$ENV{'TMP'}         = $self->{'tempdir'};
	$ENV{'TEMP'}        = $self->{'tempdir'};
	$ENV{'CORRECTION_LEVEL'} = substr($self->{'version'}, 2, 2);

	# Extract alltestpkg.sar :
	&doCmd($self, "cd $self->{'tempdir'} && SAPCAR -xvf $self->{'srcdir'}" . $self->{'delimit'} . "test" . $self->{'delimit'} . "jtest" . $self->{'delimit'} . "alltestpkg.sar");
	&doCmd($self, "cd $self->{'tempdir'} && perl jtinstall.pl -n -jr $self->{'jtest_root'} -ir $self->{'dbroot'}");
}

##################################################################
# runTF:
# run the Java TestFrame. 
##################################################################
sub runTF {
	$self = shift;
	my %tests = ();
	
	$ENV{'PATH'}        = $self->{'path'};
	$ENV{'HOME'}        = $self->{'tempdir'}; # For the PC-Tests.
	$ENV{'JTEST_ROOT'}  = $self->{'jtest_root'};
	$ENV{'INSTROOT'}    = $self->{'dbroot'};
	$ENV{'TEST_ROOT'}   = $self->{'testdir'};
	$ENV{'TESTROOT'}    = $self->{'testdir'};
	$ENV{'SAPDBSDK'}    = $self->{'dbroot'} . $self->{'delimit'} . "sdk" . $self->{'delimit'} . $self->{'version'};
	$ENV{'TMP'}         = $self->{'tempdir'};
	$ENV{'TEMP'}        = $self->{'tempdir'};
	$ENV{'CORRECTION_LEVEL'} = substr($self->{'version'}, 2, 2);

	if ($self->{'version'} eq '7402') {
		if ($self->{'profile'} eq "weekend") {
			$tests{'Alctest_weekend'}   = 'LC';
			$tests{'Blctest_recover'}   = 'LCR';
			$tests{'CDBMTest'}          = 'DBM';
			$tests{'Doltptest'}         = 'OL';
			$tests{'Eoltptest_unicode'} = 'OLU';
			$tests{'Fmut'}              = 'MU';
			$tests{'Hodbc'}             = 'OD';
			$tests{'Iodbc_unicode'}     = 'ODU';
		}
		else {
			$tests{'Alctest_night'}     = 'LC';
			$tests{'Blctest_recover'}   = 'LCR';
			$tests{'CDBMTest'}          = 'DBM';
			$tests{'Doltptest'}         = 'OL';
			$tests{'Eoltptest_unicode'} = 'OLU';
			$tests{'Fmut'}              = 'MU';
			$tests{'Hodbc'}             = 'OD';
			$tests{'Iodbc_unicode'}     = 'ODU';
		}
	}
	else {
		if ($self->{'profile'} eq "weekend") {
			$tests{'Alctest_weekend'}   = 'LC';
			$tests{'Blctest_recover'}   = 'LCR';
			$tests{'CDBMTest'}          = 'DBM';
			$tests{'Dmut'}              = 'MU';
			$tests{'Foltptest'}         = 'OL';
			$tests{'Goltptest_unicode'} = 'OLU';
			$tests{'Hodbc'}             = 'OD';
			$tests{'Iodbc_unicode'}     = 'ODU';
		}
		else {
			$tests{'Alctest_night'}     = 'LC';
			$tests{'Blctest_recover'}   = 'LCR';
			$tests{'CDBMTest'}          = 'DBM';
			$tests{'Dmut'}              = 'MU';
			$tests{'Foltptest'}         = 'OL';
			$tests{'Goltptest_unicode'} = 'OLU';
			$tests{'Hodbc'}             = 'OD';
			$tests{'Iodbc_unicode'}     = 'ODU';
		}
	}

	$myNameAppend = substr($self->{'version'}, 0, 2) . substr($self->{'version'}, 3, 1) . substr($self->{'status'}, 0, 1);
	
	while (($key, $value) = each(%tests)) {
		 $tests{$key} = $value . $myNameAppend;
	}
	
	my @testq = ();
	
	foreach $x (sort(keys(%tests))) {
		$tmp =  substr($x, 1, 1000);
		push(@testq, $tmp);
		$tests{$tmp} = $tests{$x}; 
	}
	
	
	if ($options{'verbose'}) {
		print "The following tests will be performed:\n\n";
		foreach $x (@testq) {
			print "$x\t (Testname = ${tests{$x}})\n";
		}
	}
	
	# Paralell running tests should run in reverted sequences for avoiding 
	# lock-problems with conflicting tests (this is mainly done for the 
	# oltptest-sequence)
	
	foreach $x (@testq) {
		if ($self->{'status'} ne "RAMP") {
			# Currently, we can only transmit QM-Parameters on DEV-Versions
			# Need to wait until the current DEV-Versions become COR/RAMP
			$cmd = ("cd $self->{'jtest_root'} && perl jtrun.pl -R $self->{'dbroot'} -bits $self->{'bits'} -scheduled -monitor -MAKEKEY $self->{'qah'}->{'ID'}  -QA $self->{'status'} -LCP $self->{'qah'}->{'LCPOOLID'} -sr -CL $self->{'changelist'} -d $tests{$x} $x \n");
		}
		else {
			$cmd = ("cd $self->{'jtest_root'} && perl jtrun.pl -R $self->{'dbroot'} -bits $self->{'bits'} -scheduled -monitor -sr -d $tests{$x} $x \n");
		}
		&doCmd($self, $cmd);
	}
	
	return $self->{qah}->check_lcok();
}

#
# This Method is for performing only a single test-sequence (mostly for debug-purposes)
sub run_single_test {
        # For letting the SUT run parallel
        $self = shift;
	$testname = shift;
	$dbname   = shift;


        $ENV{'PATH'}         = $self->{'path'};
        $ENV{'HOME'}         = $self->{'tempdir'}; # For the PC-Tests.
        $ENV{'JTEST_ROOT'}   = $self->{'jtest_root'};
        $ENV{'INSTROOT'}     = $self->{'dbroot'};
        $ENV{'TEST_ROOT'}    = $self->{'testdir'};
        $ENV{'TESTROOT'}     = $self->{'testdir'};
        $ENV{'SAPDBSDK'}     = $self->{'dbroot'} . $self->{'delimit'} . "sdk" . $self->{'delimit'} . $self->{'version'};
        $ENV{'TMP'}          = $self->{'tempdir'};
        $ENV{'TEMP'}         = $self->{'tempdir'};

        my %tests            = ();

        $tests{"A$testname"} = $dbname;


        $myNameAppend = substr($self->{'version'}, 0, 2) . substr($self->{'version'}, 3, 1) . substr($self->{'status'}, 0, 1);
        while (($key, $value) = each(%tests)) {
                 $tests{$key} = $value . $myNameAppend;
        }

        my @testq = ();

        foreach $x (sort(keys(%tests))) {
                $tmp =  substr($x, 1, 1000);
                push(@testq, $tmp);
                $tests{$tmp} = $tests{$x};
        }


        if ($options{'verbose'}) {
                print "The following tests will be performed:\n\n";
                foreach $x (@testq) {
                        print "$x\t (Testname = ${tests{$x}})\n";
                }
        }

        # Paralell running tests should run in reverted sequences for avoiding
        # lock-problems with conflicting tests (this is mainly done for the
        # oltptest-sequence)

        foreach $x (@testq) {
                if ($self->{'status'} ne "RAMP") {
                        # Currently, we can only transmit QM-Parameters on DEV-Versions
                        # Need to wait until the current DEV-Versions become COR/RAMP
                        $cmd = ("cd $self->{'jtest_root'} && perl jtrun.pl -R $self->{'dbroot'} -bits $self->{'bits'} -scheduled -monitor -QA $self->{'status'} -MAKEKEY $self->{'qah'}->{'ID'} -LCP $self->{'qah'}->{'LCPOOLID'} -sr -CL $self->{'changelist'} -d $tests{$x} $x \n");
                }
                else {
                        $cmd = ("cd $self->{'jtest_root'} && perl jtrun.pl -R $self->{'dbroot'} -bits $self->{'bits'} -scheduled -monitor -sr -d $tests{$x} $x \n");
                }
                &doCmd($self, $cmd);
        }
	return $self->{qah}->check_lcok();
}

sub runSUT {
	# For letting the SUT run parallel
	$self = shift;
	$ENV{'PATH'}        = $self->{'path'};
	$ENV{'HOME'}        = $self->{'tempdir'}; # For the PC-Tests.
	$ENV{'JTEST_ROOT'}  = $self->{'jtest_root'};
	$ENV{'INSTROOT'}    = $self->{'dbroot'};
	$ENV{'TEST_ROOT'}   = $self->{'testdir'};
	$ENV{'TESTROOT'}    = $self->{'testdir'};
	$ENV{'SAPDBSDK'}    = $self->{'dbroot'} . $self->{'delimit'} . "sdk" . $self->{'delimit'} . $self->{'version'};
	$ENV{'TMP'}         = $self->{'tempdir'};
	$ENV{'TEMP'}        = $self->{'tempdir'};
	$ENV{'CORRECTION_LEVEL'} = substr($self->{'version'}, 2, 2);

	my %tests = ();
	
	$tests{'Gsut'}              = 'SUT';
	
	$myNameAppend = substr($self->{'version'}, 0, 2) . substr($self->{'version'}, 3, 1) . substr($self->{'status'}, 0, 1);
	while (($key, $value) = each(%tests)) {
		 $tests{$key} = $value . $myNameAppend;
	}
	
	my @testq = ();
	
	foreach $x (sort(keys(%tests))) {
		$tmp =  substr($x, 1, 1000);
		push(@testq, $tmp);
		$tests{$tmp} = $tests{$x}; 
	}
	
	
	if ($options{'verbose'}) {
		print "The following tests will be performed:\n\n";
		foreach $x (@testq) {
			print "$x\t (Testname = ${tests{$x}})\n";
		}
	}
	
	# Paralell running tests should run in reverted sequences for avoiding 
	# lock-problems with conflicting tests (this is mainly done for the 
	# oltptest-sequence)
	
	foreach $x (@testq) {
		if ($self->{'status'} ne "RAMP") {
			# Currently, we can only transmit QM-Parameters on DEV-Versions
			# Need to wait until the current DEV-Versions become COR/RAMP
			$cmd = ("cd $self->{'jtest_root'} && perl jtrun.pl -R $self->{'dbroot'} -bits $self->{'bits'} -scheduled -monitor -QA $self->{'status'} -MAKEKEY $self->{'qah'}->{'ID'} -LCP $self->{'qah'}->{'LCPOOLID'} -sr -CL $self->{'changelist'} -d $tests{$x} $x \n");
		}
		else {
			$cmd = ("cd $self->{'jtest_root'} && perl jtrun.pl -R $self->{'dbroot'} -bits $self->{'bits'} -scheduled -monitor -sr -d $tests{$x} $x \n");
		}
		&doCmd($self, $cmd);
	}
	return $self->{qah}->check_lcok();
}

#
# Helpers
#

sub doCmd {
	my $self = shift;
	$kommando = pop(@_);
	
	my $outbuf = "CMD: \r\n $kommando \r\n";
	
	if ($options{'verbose'}) {
		print ($kommando . "\n");
	}
	
	if (!($self->{'dry'})) {
		open (CMDFILE, ($kommando . "|"));
		while (<CMDFILE>) {
			$outbuf .= $_;
		}
		close(CMDFILE);
		
		if (length($outbuf) < 3000) {
			$self->{qah}->write_log($outbuf);
			
		}
		else {
			if (length($kommando) > 299) {
				$kommando = substr($kommando, 0, 290) . "...";
			}
			
			$self->{qah}->write_prot("p" . $self->{'prot_count'} . $$, $outbuf, $kommando);
			$self->{'prot_count'}++;
		}
		if ($self->{qah}->{error_code} != 0) {
			print "Error in output-handling of $kommando \nI will ignore it.\nOutput:\n$outbuf";
			$self->{qah}->{error_code} = 0;
		}
	}
}



sub throw_err {
	my $errortext = shift;
	#my $smtp = Net::SMTP->new("mail.sap-ag.de");
	#$smtp->mail("remuser\@is0025.wdf.sap-ag.de");
	#$smtp->to("falko.flessner\@sap.com");
	#$smtp->data();
	#$smtp->datasend("To: falko.flessner\@sap.com\n");
	#$smtp->datasend("Subject: Error during qadb-run \n");
        #$smtp->datasend("Priority: Urgent\nX-Priority: 1 (Highest)\n");
	#$smtp->datasend("\n");
	#$smtp->datasend($errortext);
	#$smtp->dataend();
	#$smtp->quit;
	print $errortext;
	return $errortext;
}


