#!/usr/bin/perl -w
#
# checkmake.pl Script
#
#
#    ========== licence begin  GPL
#    Copyright (C) 2001 SAP AG
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#    ========== licence end
#
use strict;

#define vars
my ($Prot, $ErrFound, @Lines, @Msg);

foreach my $Prot ( @ARGV ) {
    
    # read prot
    if ( !open (IN, "<$Prot") )  {
        print "Make protocol $Prot not found (ignored)\n";
        next;
    }
    @Lines = <IN>;
    close IN;

    print "Checking make protocol $Prot for errors\n";
    $ErrFound = grep { /END:\s*MAKE\s*TARGET:\s*\S*\s*RESULT:\s*ERROR/i } @Lines;
    push @Msg, @Lines if ( $ErrFound );
}

if ( @Msg ) {

    print "Errors found in make protocols\n";

    # send mail
    &SendErrorMail(@Msg) if ( @Msg ); 
    # exit with error code
    if (caller()) {
        return -1;
    } else {
        exit -1;
    }
}

print "No errors found in make protocols\n";
if (caller()) {
    return 0;
} else {
    exit 0;
}

#########################################################################

sub SendErrorMail {
    use Mail::Sendmail;

    my ($msg, $sub, $receiver, $cc, %mail, $i, @targets, %t);
    my ($hostname, $rel, $qamode, $os, $tinsub, %duser, @duser);
    my (@localtime, $webdavname, $rc);

    $rel = substr($ENV{RELVER},1,1) . "." .
           substr($ENV{RELVER},2,1) . "." .
           $ENV{CORRECTION_LEVEL};
    
    $hostname =  `hostname`; 
    $hostname =~ s/\n$//;
    $os = "SUN"     if ( $hostname =~ /^u.*/i );
    $os = "DEC"     if ( $hostname =~ /^d.*/i );
    $os = "AIX"     if ( $hostname =~ /^i.*/i );
    $os = "LINUX"   if ( $hostname =~ /^l.*/i );
    $os = "SNI"     if ( $hostname =~ /^s.*/i );
    $os = "HP"      if ( $hostname =~ /^h.*/i );
    $os = "WINDOWS" if ( $hostname =~ /^p.*/i );
    
    $os .= "(IA64)" if (($os eq "WINDOWS") and ($ENV{CPU} eq "IA64"));
    
    
    @localtime = localtime(time);
    $webdavname = $os . "_" . $rel . "_" . sprintf ("%04d%02d%02d%02d%02d", $localtime[5] + 1900, $localtime[4] + 1, $localtime[3], $localtime[2], $localtime[1]); 
    open (OUT,">$webdavname");
    print OUT @_ ;
    close OUT;
    $rc = system("webdav.py $webdavname");
    unlink ($webdavname);
    
    @targets = grep { s/^TARGET:\s*'(\S*)'\s*.*ERROR.*/$1/ } @_;
    $i=0;
    foreach (@targets) {
        chomp;
        $t{$_} = ++$i unless ( defined $t{$_} );
    }
    @targets = sort {$t{$a} <=> $t{$b}} keys %t;
    
    
    $msg = "OWN = $ENV{OWN}\n\nAffected targets:\n\n" . join("\n",@targets);
    $tinsub =  join(',', @targets);
    $tinsub = substr($tinsub,0,50) . "..." if (length($tinsub) > 50);
    $qamode = $ENV{LC_STATE} ? $ENV{LC_STATE} : "DEV";
    $sub = "make problem on $os ($hostname) $rel $qamode ($tinsub)";

    $msg .= "\n\nfor more info see protfile: http://pts:1080/webdav/daily_makes/$webdavname\n";

    if ( (!defined $ENV{LC_STATE}) or ($ENV{LC_STATE} eq "DEV") ) { 
        foreach (@targets) {
            open(PIPE,"$ENV{TOOL}/bin/whose.py -format duser $_ |") or print "can't open pipe"; 
            while (<PIPE>) {
                $duser{$_} = 1 if (/d\d+/i);
            }
            close (PIPE);
        }
        @duser = keys %duser;
        @duser = grep { /^[^#]\S+$/ } <DATA> unless ( defined @duser ); 
    } else {
       @duser = ('frank.strassenburg', 'axel.mosle', 'gerald.arnold', 'falko.flessner');
    }
    
    
    $receiver = join (',', map {chomp; s/(d\d+).*/$1/i; $_ . '@exchange.sap-ag.de'} @duser);

    $cc = 'holger.becker@sap.com'    if ( $hostname =~ /^(s|u).*/i );
    $cc = 'stefan.paeck@sap.com'     if ( $hostname =~ /^d.*/i );
    $cc = 'robin.wissbrock@sap.com'  if ( $hostname =~ /^i.*/i );
    $cc = 'daniel.dittmar@sap.com'   if ( $hostname =~ /^l.*/i );
    $cc = 'gert.grossmann@sap.com'   if ( $hostname =~ /^h.*/i );
    
    if ( $hostname =~ /^p.*/i ) {
        if (($os eq "WINDOWS(IA64)")) {
            $cc = 'dennis.adler@sap.com' ;
        } else {
            $cc = 'elke.zabach@sap.com,ulrich.jansen@sap.com' ;
        }
    }
    
    %mail = ( To      => "$receiver",
              From    => 'automake@sap.com',
              Message => "$msg",
              Cc      => "$cc",
              Smtp    => 'mail.sap-ag.de',
              Subject => "$sub" );

    if ( ! sendmail(%mail) ) {
        print "FATAL: sendmail error $Mail::Sendmail::error\n";
    }
}

__DATA__
torsten.strahl
henrik.hempelmann
uwe.hahn
martin.brunzema
daniel.dittmar
martin.kittel
elke.zabach
ulrich.jansen
thomas.anhaus
ferdinand.flohe
gert.grossmann
holger.becker
joerg.mensing
raymond.roedling
stefan.paeck
robin.wissbrock
mechthild.bore
dennis.adler
robert.voelkel
ivan.schreter
markus.sinnwell
peter.goertz
steffen.schildberg
bernd.vorsprach
marco.paskamp
gerald.arnold
frank.strassenburg
